@@ -1,3 +1,9 @@
+#
+# unit_tests.py
+#
+# unit tests for littletable library
+#
+
 import unittest
 import littletable as lt
 import itertools
@@ -44,6 +50,13 @@ if PY_3:
 else:
     import StringIO as io
 
+# if rich is not installed, disable table.present() calls
+try:
+    import rich
+except ImportError:
+    lt.Table.present = lambda *args: None
+
+
 class Slotted(object):
     __slots__ = ['a', 'b', 'c']
 
@@ -65,7 +78,7 @@ class TestDataObjects(unittest.TestCase):
         ob = lt.DataObject()
         ob.z = 200
         ob.a = 100
-        self.assertEqual(sorted(ob.__dict__.items()), [('a', 100), ('z', 200)])
+        self.assertEqual([('a', 100), ('z', 200)], sorted(ob.__dict__.items()))
 
         # test semi-immutability (can't overwrite existing attributes)
         with self.assertRaises(AttributeError):
@@ -73,13 +86,13 @@ class TestDataObjects(unittest.TestCase):
 
         # equality tests
         ob2 = lt.DataObject(**{'a': 100, 'z': 200})
-        self.assertEqual(ob, ob2)
+        self.assertEqual(ob2, ob)
 
         ob2.b = 'blah'
         self.assertNotEqual(ob, ob2)
 
         del ob2.b
-        self.assertEqual(ob, ob2)
+        self.assertEqual(ob2, ob)
 
         del ob2.a
         del ob2.z
@@ -92,7 +105,7 @@ class TestDataObjects(unittest.TestCase):
         with self.assertRaises(KeyError):
             ob2['a'] = 10
 
-        self.assertEqual(repr(ob2), "{'a': 10}")
+        self.assertEqual("{'a': 10}", repr(ob2))
 
 class TestTableTypes(unittest.TestCase):
     def test_types(self):
@@ -157,7 +170,7 @@ class TableCreateTests:
         table.insert(self.make_data_object(1, 2, 3))
         table.insert(self.make_data_object(4, 5, 6))
         table.create_index('a', unique=True)
-        self.assertEqual(table.by.a[4], self.make_data_object(4, 5, 6))
+        self.assertEqual(self.make_data_object(4, 5, 6), table.by.a[4])
 
         with self.assertRaises(KeyError):
             table.insert(self.make_data_object(4, 1, 0))
@@ -175,47 +188,57 @@ class TableCreateTests:
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
 
-        self.assertEqual(len(table.where(a=5)), test_size*test_size)
-        self.assertEqual(len(table.where(a=-1)), 0)
+        self.assertEqual(test_size*test_size, len(table.where(a=5)))
+        self.assertEqual(0, len(table.where(a=-1)))
 
     def test_where_equals_none(self):
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
 
-        self.assertEqual(len(table.where(a=5, b=test_size)), 0)
+        self.assertEqual(0, len(table.where(a=5, b=test_size)))
 
     def test_where_equals_with_index(self):
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
         table.create_index('a')
 
-        self.assertEqual(len(table.where(a=5)), test_size*test_size)
-        self.assertEqual(len(table.where(a=-1)), 0)
+        self.assertEqual(test_size*test_size, len(table.where(a=5)))
+        self.assertEqual(0, len(table.where(a=-1)))
 
     def test_where_range(self):
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
 
-        self.assertEqual(len(table.where(lambda rec: rec.a == rec.b)), test_size*test_size)
+        self.assertEqual(test_size*test_size, len(table.where(lambda rec: rec.a == rec.b)))
 
     def test_where_comparator(self):
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
 
-        self.assertEqual(len(table.where(a=lt.Table.lt(4))), test_size*test_size*4)
-        self.assertEqual(len(table.where(a=lt.Table.le(4))), test_size*test_size*(4+1))
-        self.assertEqual(len(table.where(a=lt.Table.gt(4))), test_size*test_size*(test_size-4-1))
-        self.assertEqual(len(table.where(a=lt.Table.ge(4))), test_size*test_size*(test_size-4))
-        self.assertEqual(len(table.where(a=lt.Table.ne(4))), test_size*test_size*(test_size-1))
-        self.assertEqual(len(table.where(a=lt.Table.eq(4))), test_size*test_size)
-        self.assertEqual(len(table.where(a=lt.Table.eq(4), b=lt.Table.eq(4))), test_size)
+        self.assertEqual(test_size*test_size*4, len(table.where(a=lt.Table.lt(4))))
+        self.assertEqual(test_size*test_size*(4+1), len(table.where(a=lt.Table.le(4))))
+        self.assertEqual(test_size*test_size*(test_size-4-1), len(table.where(a=lt.Table.gt(4))))
+        self.assertEqual(test_size*test_size*(test_size-4), len(table.where(a=lt.Table.ge(4))))
+        self.assertEqual(test_size*test_size*(test_size-1), len(table.where(a=lt.Table.ne(4))))
+        self.assertEqual(test_size*test_size, len(table.where(a=lt.Table.eq(4))))
+        self.assertEqual(test_size, len(table.where(a=lt.Table.eq(4), b=lt.Table.eq(4))))
+        self.assertEqual(test_size*test_size*4, len(table.where(a=lt.Table.between(3, 8))))
+        self.assertEqual(test_size*test_size*4, len(table.where(a=lt.Table.within(2, 5))))
+        self.assertEqual(test_size*test_size*3, len(table.where(a=lt.Table.in_range(2, 5))))
+        self.assertEqual(0, len(table.where(a=lt.Table.between(3, 3))))
+        self.assertEqual(test_size*test_size, len(table.where(a=lt.Table.within(3, 3))))
+        self.assertEqual(0, len(table.where(a=lt.Table.in_range(3, 3))))
+        self.assertEqual(test_size*test_size*4, len(table.where(a=lt.Table.is_in([2, 4, 6, 8]))))
+        self.assertEqual(0, len(table.where(a=lt.Table.is_in([]))))
+        self.assertEqual(test_size*test_size*(test_size-4), len(table.where(a=lt.Table.not_in([2, 4, 6, 8]))))
+        self.assertEqual(test_size*test_size*test_size, len(table.where(a=lt.Table.not_in([]))))
 
     def test_get_slice(self):
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
 
         subtable = table[0::test_size]
-        self.assertEqual(len(subtable), test_size * test_size)
+        self.assertEqual(test_size * test_size, len(subtable))
 
     def test_indexing(self):
         chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -226,12 +249,12 @@ class TableCreateTests:
 
         self.assertTrue('A' in table.by.a)
         self.assertTrue('AA' not in table.by.a)
-        self.assertEqual(len(table.by.a['B']), test_size * test_size)
+        self.assertEqual(test_size * test_size, len(table.by.a['B']))
         self.assertTrue(isinstance(table.by.a['B'], lt.Table))
         with self.assertRaises(AttributeError):
             table.by.z
 
-        self.assertEqual(len(table.by.a.keys()), test_size)
+        self.assertEqual(test_size, len(table.by.a.keys()))
 
     def test_unique_indexing(self):
         chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -272,10 +295,10 @@ class TableCreateTests:
         import pprint
         info = table.info()
         pprint.pprint(info)
-        self.assertEqual(info['name'], 'Table_1')
-        self.assertEqual(list(sorted(info['fields'])), list('abc'))
-        self.assertEqual(list(sorted(info['indexes'])), [('a', True), ('c', False)])
-        self.assertEqual(info['len'], 1001)
+        self.assertEqual('Table_1', info['name'])
+        self.assertEqual(['a', 'b', 'c'], list(sorted(info['fields'])))
+        self.assertEqual([('a', True), ('c', False)], list(sorted(info['indexes'])))
+        self.assertEqual(1001, info['len'])
 
     def test_chained_indexing(self):
         chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -313,21 +336,20 @@ class TableCreateTests:
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
 
-        self.assertEqual(table.delete(b=5), test_size*test_size)
-        self.assertEqual(len(table), test_size*test_size*(test_size-1))
-        self.assertEqual(table.delete(b=-1), 0)
-
-        self.assertEqual(table.delete(), 0)
+        self.assertEqual(test_size*test_size, table.delete(b=5))
+        self.assertEqual(test_size*test_size*(test_size-1), len(table))
+        self.assertEqual(0, table.delete(b=-1))
+        self.assertEqual(0, table.delete())
 
     def test_remove_many(self):
         test_size = 10
         table = make_test_table(self.make_data_object, test_size)
 
         table.remove_many(table.where(lambda t: t.a % 2))
-        self.assertEqual(len(table), test_size*test_size*test_size/2)
+        self.assertEqual(test_size*test_size*test_size/2, len(table))
         table_len = len(table)
         table.remove(table[1])
-        self.assertEqual(len(table), table_len-1)
+        self.assertEqual(table_len-1, len(table))
 
     def test_add_new_field(self):
         test_size = 10
@@ -338,7 +360,7 @@ class TableCreateTests:
             table.add_field('d', lambda rec: rec.a+rec.b+rec.c)
 
             table.create_index('d')
-            self.assertEqual(len(table.by.d.keys()), len(range(0, 27+1)))
+            self.assertEqual(len(range(0, 27+1)), len(table.by.d.keys()))
 
     def test_add_two_tables(self):
         test_size = 10
@@ -346,15 +368,15 @@ class TableCreateTests:
         make_rec = lambda a,b,c: self.make_data_object(a+test_size, b, c)
         t2 = make_test_table(make_rec, test_size)
 
-        self.assertEqual(len(t1+t2), test_size*test_size*test_size*2)
-        self.assertEqual(len(t1), test_size * test_size * test_size)
+        self.assertEqual(test_size*test_size*test_size*2, len(t1+t2))
+        self.assertEqual(test_size * test_size * test_size, len(t1))
 
         t1 += t2
-        self.assertEqual(len(t1), test_size * test_size * test_size * 2)
+        self.assertEqual(test_size * test_size * test_size * 2, len(t1))
 
         offset = test_size * test_size
         t3 = t1 + (self.make_data_object(rec.a+offset, rec.b, rec.c) for rec in t2)
-        self.assertEqual(len(t3), test_size * test_size * test_size * 3)
+        self.assertEqual(test_size * test_size * test_size * 3, len(t3))
 
     def test_table_info(self):
         test_size = 10
@@ -364,10 +386,11 @@ class TableCreateTests:
         # must sort fields and indexes values, for test comparisons
         t1_info['fields'].sort()
         t1_info['indexes'].sort()
-        self.assertEqual(t1_info, {'fields': ['a', 'b', 'c'],
-                                     'indexes': [('b', False)],
-                                     'len': 1000,
-                                     'name': 'info_test'}, "invalid info results")
+        self.assertEqual({'fields': ['a', 'b', 'c'],
+                          'indexes': [('b', False)],
+                          'len': 1000,
+                          'name': 'info_test'},
+                         t1_info, "invalid info results")
 
 
 class TableCreateTests_DataObjects(unittest.TestCase, TableCreateTests, UsingDataObjects):
@@ -399,7 +422,7 @@ class TableListTests:
 
     def test_index_find(self):
         self._test_init()
-        self.assertEqual(self.t1.index(self.test_rec), 13, "failed 'in' (contains) test")
+        self.assertEqual(13, self.t1.index(self.test_rec), "failed 'in' (contains) test")
 
     def test_remove(self):
         self._test_init()
@@ -425,22 +448,82 @@ class TableListTests:
         self._test_init()
         self.assertTrue(self.test_rec in self.t1, "failed 'in' (contains) test")
 
+    def test_head_and_tail(self):
+        self._test_init()
+        self.t1.create_index("a")
+        self.t1.create_index("c")
+        self.assertEqual(set(["a", "c"]), set(self.t1._indexes.keys()), "failed to create indexes")
+
+        self.assertEqual(set(self.t1._indexes.keys()),
+                         set(self.t1.head()._indexes.keys()),
+                         "failed to copy indexes to head()")
+        self.assertEqual(set(self.t1._indexes.keys()),
+                         set(self.t1.tail()._indexes.keys()),
+                         "failed to copy indexes to tail()")
+
     def test_unique(self):
         self._test_init()
-        self.assertEqual(sorted([row.a for row in self.t1.unique('a')]), [0, 1, 2], "failed call to unique")
+        self.assertEqual([0, 1, 2], sorted([row.a for row in self.t1.unique('a')]), "failed call to unique")
 
     def test_all_accessor(self):
         self._test_init()
-        self.assertEqual(list(self.t1.all.a),
-                         sum(([i]*self.test_size**2 for i in range(self.test_size)), []),
+        self.assertEqual(sum(([i]*self.test_size**2 for i in range(self.test_size)), []),
+                         list(self.t1.all.a),
                          "failed to successfully get all values in 'a'")
 
+        all_as = self.t1.all.a
+        self.assertTrue(all_as is iter(all_as), "all iterator fails to identify as iter(self)")
+
     def test_format(self):
         self._test_init()
-        self.assertEqual(list(self.t1.format("{a:02d} {b} {c}"))[:3],
-                         ['00 0 0', '00 0 1', '00 0 2'],
+        self.assertEqual(['00 0 0', '00 0 1', '00 0 2'],
+                         list(self.t1.format("{a:02d} {b} {c}"))[:3],
                          "failed to create formatted rows")
 
+    def test_as_html(self):
+        self._test_init()
+        html_output = self.t1[:10].as_html(fields="a b c")
+        print(html_output)
+        self.assertTrue("<thead>" in html_output and "<tbody>" in html_output,
+                        "as_html does not include thead and tbody tags")
+
+        html_lines = html_output.splitlines()
+        hdr_line = next(h for h in html_lines if "center" in h)
+        self.assertEqual('<tr><th><div align="center">a</div></th>'
+                         '<th><div align="center">b</div></th>'
+                         '<th><div align="center">c</div></th></tr>',
+                         hdr_line,
+                         "failed as_html with all fields")
+
+        html_output = self.t1[:10].as_html(fields="a -b c")
+        print(html_output)
+        html_lines = html_output.splitlines()
+        hdr_line = next(h for h in html_lines if "center" in h)
+        self.assertEqual('<tr><th><div align="center">a</div></th>'
+                         '<th><div align="center">c</div></th></tr>',
+                         hdr_line,
+                         "failed as_html with negated field")
+
+        html_output = self.t1[:10].as_html(fields="a b c", formats={"b": "{:03d}"})
+        print(html_output)
+        html_lines = html_output.splitlines()
+        data_line = next(h for h in html_lines if "<td>" in h)
+        self.assertEqual('<tbody><tr><td><div align="right">0</div></td>'
+                         '<td><div align="right">000</div></td>'
+                         '<td><div align="right">0</div></td></tr>',
+                         data_line,
+                         "failed as_html with named field format")
+
+        html_output = self.t1[:10].as_html(fields="a b c", formats={int: "{:03d}"})
+        print(html_output)
+        html_lines = html_output.splitlines()
+        data_line = next(h for h in html_lines if "<td>" in h)
+        self.assertEqual('<tbody><tr><td><div align="right">000</div></td>'
+                         '<td><div align="right">000</div></td>'
+                         '<td><div align="right">000</div></td></tr>',
+                         data_line,
+                         "failed as_html with data type format")
+
     def test_delete_slices(self):
         compare_list = list("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz")
         t1 = lt.Table().insert_many(lt.DataObject(A=c) for c in compare_list)
@@ -478,6 +561,52 @@ class TableListTests:
         mini_test((1000, 2000, None))
         mini_test((None, None, -1))
 
+    def test_clear(self):
+        self._test_init()
+        num_fields = len(self.t1.info()["fields"])
+        self.assertEqual(self.test_size ** num_fields, len(self.t1), "invalid len")
+        self.t1.create_index("a")
+
+        self.t1.clear()
+        self.assertEqual(0, len(self.t1), "invalid len after clear")
+        self.assertEqual(1, len(self.t1.info()["indexes"]), "invalid indexes after clear")
+
+    def test_stats(self):
+        self._test_init()
+        field_names = self.t1.info()["fields"]
+        num_fields = len(field_names)
+        t1_stats = self.t1.stats().select("name count min max mean")
+        for fieldname in field_names:
+            stat_rec = t1_stats.by.name[fieldname]
+            self.assertEqual(lt.DataObject(name=fieldname,
+                                           count=self.test_size ** num_fields,
+                                           min=0,
+                                           max=self.test_size - 1,
+                                           mean=(self.test_size - 1) / 2),
+                             stat_rec,
+                             "invalid stat for {}".format(fieldname))
+
+        t1_stats = self.t1.stats(by_field=False)
+        for stat, value in (('min', 0), ('max', self.test_size - 1), ('count', self.test_size ** num_fields),):
+            for fieldname in field_names:
+                self.assertEqual(value, getattr(t1_stats.by.stat[stat], fieldname),
+                                 "invalid {} stat for {}".format(stat, fieldname))
+
+        # verify that stats can "step over" non-numeric data
+        try:
+            self.t1[0].a = "not a number"
+        except (AttributeError, TypeError):
+            # some test types aren't mutable, must replace rec with a modified one
+            mod_rec = self.t1.pop(0)
+            rec_type = type(mod_rec)
+            new_rec_dict = lt._to_dict(mod_rec)
+            new_rec_dict['a'] = "not a number"
+            new_rec = rec_type(**new_rec_dict)
+            self.t1.insert(new_rec)
+
+        t1_stats = self.t1.stats()
+        self.assertEqual(self.test_size ** num_fields - 1, t1_stats.by.name["a"].count)
+
 class TableListTests_DataObjects(unittest.TestCase, TableListTests, UsingDataObjects):
     pass
 
@@ -506,13 +635,13 @@ class TableJoinTests:
         t2.insert(lt.DataObject(a=1, d=100))
 
         joined = (t1.join_on('a') + t2.join_on('a'))()
-        self.assertEqual(len(joined), test_size * test_size)
+        self.assertEqual(test_size * test_size, len(joined))
 
         joined = (t1.join_on('a') + t2)()
-        self.assertEqual(len(joined), test_size * test_size)
+        self.assertEqual(test_size * test_size, len(joined))
 
         joined = (t1 + t2.join_on('a'))()
-        self.assertEqual(len(joined), test_size * test_size)
+        self.assertEqual(test_size * test_size, len(joined))
 
         t1.delete_index('a')
         with self.assertRaises(ValueError):
@@ -527,7 +656,7 @@ class TableJoinTests:
             t3 = t1.join(t2, 'a,d,z', a='a')
 
         t3 = t1.join(t2, 'a,d', a='a')
-        self.assertEqual(len(t3), test_size * test_size)
+        self.assertEqual(test_size * test_size, len(t3))
 
         t4 = t1.join(t2, a='a').select('a c d', e=lambda rec: rec.a + rec.c + rec.d)
         self.assertTrue(all(rec.e == rec.a+rec.c+rec.d for rec in t4))
@@ -536,7 +665,113 @@ class TableJoinTests:
         empty_table = lt.Table()
         empty_table.create_index('a')
         t5 = (t1.join_on('a') + empty_table)()
-        self.assertEqual(len(t5), 0)
+        self.assertEqual(0, len(t5))
+
+    def test_outer_joins(self):
+        t1 = lt.Table("catalog")
+        t1.csv_import(textwrap.dedent("""\
+            sku,color,size,material
+            001,red,XL,cotton
+            002,blue,XL,cotton/poly
+            003,blue,L,linen
+            004,red,M,cotton
+            """))
+
+        t2 = lt.Table("prices")
+        t2.csv_import(textwrap.dedent("""\
+            sku,unit_price,size
+            001,10,L
+            001,12,XL
+            002,11,
+            004,9,
+            """), transforms={'size': lambda x: x or None})
+        print(t1.info())
+
+        t1.present()
+        t2.present()
+
+        t3 = t1.join(t2, auto_create_indexes=True, sku="sku")
+        print(t3.info())
+        t3.present()
+        self.assertEqual(4, len(t3))
+
+        t3 = t1.join(t2, auto_create_indexes=True, sku="sku", size="size")
+        t3("inner join - " + t3.table_name)
+        print(t3.info())
+        t3.present()
+        self.assertEqual(1, len(t3))
+
+        t3 = t1.outer_join(lt.Table.RIGHT_OUTER_JOIN, t2, sku="sku", size="size")
+        t3("right outer join - " + t3.table_name)
+        print(t3.info())
+        t3.present()
+        self.assertEqual(4, len(t3))
+
+        t3 = t1.outer_join(lt.Table.LEFT_OUTER_JOIN, t2, sku="sku", size="size")
+        t3("left outer join - " + t3.table_name)
+        print(t3.info())
+        t3.present()
+        self.assertEqual(2, len(t3))
+
+        t3 = t1.outer_join(lt.Table.FULL_OUTER_JOIN, t2, sku="sku", size="size")
+        t3("full outer join - " + t3.table_name)
+        print(t3.info())
+        t3.present()
+        self.assertEqual(12, len(t3))
+
+    def test_outer_join_example(self):
+        # define student and registration data
+        students = lt.Table("students").csv_import(textwrap.dedent("""\
+            student_id,name
+            0001,Alice
+            0002,Bob
+            0003,Charlie
+            0004,Dave
+            0005,Enid
+            """))
+
+        registrations = lt.Table("registrations").csv_import(textwrap.dedent("""\
+            student_id,course
+            0001,PSYCH101
+            0001,CALC1
+            0003,BIO200
+            0005,CHEM101
+            0006,PHY101
+            """))
+
+        courses = lt.Table("courses").csv_import(textwrap.dedent("""\
+            course
+            BIO200
+            CALC1
+            CHEM101
+            PSYCH101
+            PE101
+            """))
+
+        # perform outer join and show results:
+        non_reg = students.outer_join(lt.Table.RIGHT_OUTER_JOIN,
+                                      registrations,
+                                      student_id="student_id").where(course=None)
+        non_reg.present()
+        print(list(non_reg.all.name))
+        self.assertEqual(['Bob', 'Dave'], sorted(non_reg.all.name))
+
+        # courses with no students
+        no_students = registrations.outer_join(lt.Table.LEFT_OUTER_JOIN,
+                                         courses,
+                                         course="course").where(student_id=None)
+        no_students.present()
+        print(list(no_students.all.course))
+        self.assertEqual(['PE101'], sorted(no_students.all.course))
+
+
+        full =  students.outer_join(lt.Table.FULL_OUTER_JOIN,
+                                      registrations,
+                                      student_id="student_id").where(lambda rec: rec.course is None
+                                                                                 or rec.name is None)
+        full.present()
+        print(sorted(full.all.student_id))
+        self.assertEqual(['0002', '0004', '0006'], sorted(full.all.student_id))
 
 class TableJoinTests_DataObjects(unittest.TestCase, TableJoinTests, UsingDataObjects):
     pass
@@ -564,18 +799,18 @@ class TableTransformTests:
         for c_value, recs in itertools.groupby(t1, key=lambda rec: rec.c):
             c_groups += 1
             list(recs)
-        self.assertEqual(c_groups, test_size * test_size * test_size)
+        self.assertEqual(test_size * test_size * test_size, c_groups)
 
         t1.sort('c')
         c_groups = 0
         for c_value, recs in itertools.groupby(t1, key=lambda rec: rec.c):
             c_groups += 1
             list(recs)
-        self.assertEqual(c_groups, test_size)
-        self.assertEqual(t1[0].c, 0)
+        self.assertEqual(test_size, c_groups)
+        self.assertEqual(0, t1[0].c)
 
         t1.sort('c desc')
-        self.assertEqual(t1[0].c, test_size-1)
+        self.assertEqual(test_size-1, t1[0].c)
 
     def test_sort2(self):
 
@@ -655,10 +890,10 @@ class TableTransformTests:
         t1 = make_test_table(self.make_data_object, test_size)
 
         t2 = t1.unique()
-        self.assertEqual(len(t2), len(t1))
+        self.assertEqual(len(t1), len(t2))
 
         t3 = t1.unique(key=lambda rec: rec.c)
-        self.assertEqual(len(t3), test_size)
+        self.assertEqual(test_size, len(t3))
 
 class TableTransformTests_DataObjects(unittest.TestCase, TableTransformTests, UsingDataObjects):
     pass
@@ -784,7 +1019,7 @@ class TableImportExportTests:
             outlines = out.read().splitlines()
             out.close()
             self.assertEqual(','.join(fieldnames), outlines[0])
-            self.assertEqual(len(outlines), test_size**3+1)
+            self.assertEqual(test_size**3+1, len(outlines))
             for ob, line in zip(t1, outlines[1:]):
                 csv_vals = line.split(',')
                 self.assertTrue(all(
@@ -800,7 +1035,7 @@ class TableImportExportTests:
             outlines = out.read().splitlines()
             out.close()
             self.assertEqual(','.join(fieldnames), outlines[0])
-            self.assertEqual(len(outlines), 1)
+            self.assertEqual(1, len(outlines))
 
         # rerun using an empty table, with indexes to dictate fieldnames
         for fieldnames in permutations(list('abc')):
@@ -812,8 +1047,8 @@ class TableImportExportTests:
             out.seek(0)
             outlines = out.read().splitlines()
             out.close()
-            self.assertEqual(','.join(fieldnames), outlines[0])
-            self.assertEqual(len(outlines), 1)
+            self.assertEqual(set(fieldnames), set(outlines[0].split(',')))
+            self.assertEqual(1, len(outlines))
 
     def test_csv_import(self):
         data = csv_data
@@ -834,6 +1069,53 @@ class TableImportExportTests:
         print(type(csvtable2[0]).__name__, csvtable2[0])
         self.assertEqual(type(t1[0]), type(csvtable2[0]))
 
+    def test_csv_compressed_import(self):
+        tt = lt.Table().csv_import("test/abc.csv", transforms=dict.fromkeys("abc", int))
+        print("abc.csv", tt.info())
+
+        compressed_files = ["abc.csv.zip", "abc.csv.gz"]
+        if PY_3:
+            compressed_files.append("abc.csv.xz")
+        for name in compressed_files:
+            tt2 = lt.Table().csv_import("test/" + name, transforms=dict.fromkeys("abc", int))
+            print(name, tt2.info())
+            self.assertEqual(tt.info(), tt2.info())
+            self.assertEqual(sum(tt.all.a), sum(tt2.all.a))
+            self.assertEqual(sum(tt.all.b), sum(tt2.all.b))
+            self.assertEqual(sum(tt.all.c), sum(tt2.all.c))
+
+        if PY_2:
+            with self.assertRaises(Exception):
+                name = "abc.csv.xz"
+                tt2 = lt.Table().csv_import("test/" + name, transforms=dict.fromkeys("abc", int))
+
+        tt2 = lt.Table().json_import("test/abc.json.gz")
+        print("abc.json.gz", tt2.info())
+        self.assertEqual(tt.info(), tt2.info())
+        self.assertEqual(sum(tt.all.a), sum(tt2.all.a))
+        self.assertEqual(sum(tt.all.b), sum(tt2.all.b))
+        self.assertEqual(sum(tt.all.c), sum(tt2.all.c))
+
+    def test_csv_filtered_import(self):
+        test_size = 3
+        tt = lt.Table().csv_import("test/abc.csv", transforms=dict.fromkeys("abc", int))
+        print("abc.csv", tt.info())
+
+        tt = lt.Table().csv_import("test/abc.csv", transforms=dict.fromkeys("abc", int),
+                                   filters={"c": lt.Table.eq(1)})
+        print(tt.info())
+        self.assertEqual(test_size * test_size, len(tt))
+
+        tt = lt.Table().csv_import("test/abc.csv", transforms=dict.fromkeys("abc", int),
+                                   filters={"c": 1})
+        print(tt.info())
+        self.assertEqual(test_size * test_size, len(tt))
+
+        tt = lt.Table().csv_import("test/abc.csv", transforms=dict.fromkeys("abc", int),
+                                   filters={"c": lambda x: 0 < x < 2})
+        print(tt.info())
+        self.assertEqual(test_size * test_size, len(tt))
+
     def test_csv_string_import(self):
         data = csv_data
         csvtable = lt.Table().csv_import(csv_source=data, transforms={'a': int, 'b': int, 'c': int})
@@ -894,12 +1176,12 @@ class TableImportExportTests:
             outlines = out.read().splitlines()
             out.close()
 
-            self.assertEqual(len(outlines), test_size**3)
+            self.assertEqual(test_size**3, len(outlines))
 
             for ob, line in zip(t1, outlines):
                 json_dict = json.loads(line)
                 t1_dataobj = make_dataobject_from_ob(ob)
-                self.assertEqual(lt.DataObject(**json_dict), t1_dataobj)
+                self.assertEqual(t1_dataobj, lt.DataObject(**json_dict))
 
     def test_json_import(self):
         data = json_data
